﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MySql.Data.MySqlClient;
using System.Data;

namespace MySqlConnector
{
    /// <summary>
    /// MySqlConnectorサンプルクラス
    /// </summary>
    public class MySqlClient
    {
        private MySqlConnection connection;
        private MySqlTransaction transaction;

        #region コンストラクタ

        /// <summary>
        /// MySqlClientのインスタンスを生成します
        /// </summary>
        public MySqlClient() { }

        #endregion

        #region static

        /// <summary>
        /// 接続文字列を作成します
        /// </summary>
        /// <returns>接続文字列</returns>
        public static string CreateConnectionString(params string[] args)
        {
            return string.Join(";", args);
        }

        /// <summary>
        /// 接続文字列を作成します
        /// </summary>
        /// <param name="server">サーバ名</param>
        /// <param name="database">データベース名</param>
        /// <param name="userId">ユーザID</param>
        /// <param name="password">パスワード</param>
        /// <returns>接続文字列</returns>
        public static string CreateConnectionString(string server, string database, string userId, string password)
        {
            MySqlConnectionStringBuilder builder = new MySqlConnectionStringBuilder();
            builder.Server = server;
            builder.Database = database;
            builder.UserID = userId;
            builder.Password = password;
            //builder.IgnorePrepare = false;

            return builder.ToString();
        }

        #endregion

        #region 接続開始、終了

        /// <summary>
        /// MySQLに接続します
        /// </summary>
        /// <param name="connectionString">接続文字列</param>
        public MySqlResult Connect(string connectionString)
        {
            try {
                // インスタンス未生成時または非接続時のみOpenメソッドを呼び出し
                if ((connection == null) || (connection.State == ConnectionState.Closed)) {
                    connection = new MySqlConnection(connectionString);
                    connection.Open();
                }
            } catch (Exception exception) {
                return this.ReturnErrorResult(exception);
            }

            return new MySqlResult(true);
        }

        /// <summary>
        /// MySQLとの接続を閉じます
        /// </summary>
        public MySqlResult Close()
        {
            try {
                // 接続時のみCloseメソッドを呼び出し
                if ((connection != null) && (connection.State != ConnectionState.Closed))
                    connection.Close();
            } catch (Exception exception) {
                return this.ReturnErrorResult(exception);
            }

            return new MySqlResult(true);
        }

        /// <summary>
        /// エラー結果を返却します
        /// </summary>
        /// <param name="exception">Exception</param>
        private MySqlResult ReturnErrorResult(Exception exception)
        {
            MySqlResult result = new MySqlResult(false);
            // メッセージ設定
            result.ErrorMessages = new string[] { exception.Message, exception.StackTrace };
            return result;
        }

        #endregion

        #region トランザクション

        /// <summary>
        /// トランザクションを開始します
        /// </summary>
        /// <param name="level">分離レベル</param>
        public MySqlResult BeginTransaction(IsolationLevel level)
        {
            try {
                if ((State == ConnectionState.Open) && (this.transaction == null))
                    transaction = connection.BeginTransaction(level);
            } catch (Exception exception) {
                return ReturnErrorResult(exception);
            }

            return new MySqlResult(true);
        }

        /// <summary>
        /// トランザクションを開放します
        /// </summary>
        public void DisposeTransaction()
        {
            if (transaction != null)
                transaction.Dispose();
            transaction = null;
        }

        #endregion

        #region コミット、ロールバック

        /// <summary>
        /// コミットします
        /// </summary>
        public MySqlResult Commit()
        {
            try {
                transaction.Commit();
            } catch (Exception exception) {
                return ReturnErrorResult(exception);
            }

            return new MySqlResult(true);
        }

        /// <summary>
        /// ロールバックします
        /// </summary>
        public MySqlResult Rollback()
        {
            try {
                transaction.Rollback();
            } catch (Exception exception) {
                return ReturnErrorResult(exception);
            }

            return new MySqlResult(true);
        }

        #endregion

        #region select

        /// <summary>
        /// 検索結果を取得します
        /// </summary>
        /// <param name="sql">SQL文</param>
        /// <param name="useTransaction">トランザクション使用有無</param>
        public MySqlResult<DataTable> GetSearchResultTable(string sql, bool useTransaction)
        {
            MySqlResult<DataTable> result = new MySqlResult<DataTable>();

            try {
                // SQL文を設定
                MySqlCommand command = useTransaction ?
                    new MySqlCommand(sql, connection, transaction) : new MySqlCommand(sql, connection);
                MySqlDataAdapter adapater = new MySqlDataAdapter(command);
                DataTable table = new DataTable();
                adapater.Fill(table);
                result.Value = table;
            } catch (Exception exception) {
                // エラーメッセージ設定
                result.IsSuccess = false;
                result.ErrorMessages = new string[] { exception.Message, exception.StackTrace };
                return result;
            }

            result.IsSuccess = true;
            return result;
        }

        /// <summary>
        /// 検索結果を取得します
        /// </summary>
        /// <param name="sql">SQL文</param>
        /// <param name="useTransaction">トランザクション使用有無</param>
        public MySqlResult<MySqlDataReader> GetSearchResultReader(string sql, bool useTransaction)
        {
            MySqlResult<MySqlDataReader> result = new MySqlResult<MySqlDataReader>();

            try {
                // SQL文を設定
                MySqlCommand command = useTransaction ?
                    new MySqlCommand(sql, connection, transaction) : new MySqlCommand(sql, connection);
                result.Value = command.ExecuteReader();
            } catch (Exception exception) {
                // エラーメッセージ設定
                result.IsSuccess = false;
                result.ErrorMessages = new string[] { exception.Message, exception.StackTrace };
                return result;
            }

            result.IsSuccess = true;
            return result;
        }

        #endregion

        #region insert、update、delete

        /// <summary>
        /// データを挿入します
        /// </summary>
        /// <param name="sql">SQL文</param>
        /// <param name="useTransaction">トランザクション使用有無</param>
        public MySqlResult InsertData(string sql, bool useTransaction)
        {
            return ExecuteSql(sql, useTransaction);
        }

        /// <summary>
        /// データを更新します
        /// </summary>
        /// <param name="sql">SQL文</param>
        /// <param name="useTransaction">トランザクション使用有無</param>
        public MySqlResult UpdateData(string sql, bool useTransaction)
        {
            return ExecuteSql(sql, useTransaction);
        }

        /// <summary>
        /// データを削除します
        /// </summary>
        /// <param name="sql">SQL文</param>
        /// <param name="useTransaction">トランザクション使用有無</param>
        public MySqlResult DeleteData(string sql, bool useTransaction)
        {
            return ExecuteSql(sql, useTransaction);
        }

        /// <summary>
        /// SQLを実行します
        /// </summary>
        /// <param name="sql">SQL文</param>
        /// <param name="useTransaction">トランザクション使用有無</param>
        public MySqlResult ExecuteSql(string sql, bool useTransaction)
        {
            try {
                MySqlCommand command = useTransaction ?
                    new MySqlCommand(sql, connection, transaction) : new MySqlCommand(sql, connection);
                command.ExecuteNonQuery();
            } catch (Exception exception) {
                return ReturnErrorResult(exception);
            }

            return new MySqlResult(true);
        }

        #endregion

        /// <summary>
        /// 現在の接続状態を取得します
        /// </summary>
        public ConnectionState State
        {
            get { return (connection == null) ? ConnectionState.Closed : connection.State; }
        }
    }
}
